@@ -1,5 +1,4 @@
 import numpy as np
-import unittest
 import sys
 sys.path.append("tests/python")
 # Don't import the test class, otherwise they will run twice.
@@ -7,7 +6,7 @@ import test_interaction_constraints as test_ic  # noqa
 rng = np.random.RandomState(1994)
 
 
-class TestGPUInteractionConstraints(unittest.TestCase):
+class TestGPUInteractionConstraints:
     cputest = test_ic.TestInteractionConstraints()
 
     def test_interaction_constraints(self):
